
import numpy as np
from message_worker import message_worker
import torch
import torch.distributed.rpc as rpc
import torch.optim as optim
from torch.distributed.rpc import RRef, rpc_async, remote
from torch.distributions import Categorical

from Sample.Sampler import NeighborSampler


class Agent:
    
    def __init__(self, worker_size,OBSERVER_NAME):
        self.ob_rrefs = []
        self.agent_rref = RRef(self)
        self.rewards = {}
        self.saved_log_probs = {}
        self.worker_size=worker_size
        for ob_rank in range(worker_size):
            ob_info = rpc.get_worker_info(OBSERVER_NAME.format(ob_rank))
            print(OBSERVER_NAME.format(ob_rank),(worker_size,OBSERVER_NAME))
            self.ob_rrefs.append(remote(ob_info, message_worker,args=(worker_size,OBSERVER_NAME)))
            self.rewards[ob_info.id] = []
            self.saved_log_probs[ob_info.id] = []
        
    def run_sample(self,batch_size,num_samples):
        futs=[]    
        sampler=NeighborSampler()
        for ob_rref in self.ob_rrefs:
            futs.append(
                rpc_async(
                    ob_rref.owner(),
                    ob_rref.rpc_sync().sampler,
                    args=(self.ob_rrefs,sampler,5),
                    kwargs={"num_neighbors":3}
                )
            )
        for fut in futs:
            fut.wait()

    def loadGraph(self,path):
        print((path))
        futs=[]
        for ob_rref in self.ob_rrefs:
            futs.append(
                rpc_async(
                    ob_rref.owner(),
                    ob_rref.rpc_sync().loadGraph,
                    args=(path,)
                )
            )
        for fut in futs:
            fut.wait()
        
   